"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ab = require("asyncblock");
var fs = require("fs");
var protractor_1 = require("protractor");
var selenium_webdriver_1 = require("selenium-webdriver");
var browser_sync_1 = require("./browser-sync");
var config_1 = require("./config");
var exec_1 = require("./exec");
var polled_wait_1 = require("./polled-wait");
var selection_1 = require("./selection");
var vars_1 = require("./vars");
var ElementFinderSync = (function () {
    function ElementFinderSync(element) {
        this.element = element;
    }
    ElementFinderSync.prototype.findVisible = function (selector) {
        return selection_1.findVisible(selector, this);
    };
    ElementFinderSync.prototype.findVisibles = function (selector) {
        return selection_1.findVisibles(selector, this);
    };
    ElementFinderSync.prototype.findElement = function (selector) {
        return selection_1.findElement(selector, this);
    };
    ElementFinderSync.prototype.findElements = function (selector) {
        return selection_1.findElements(selector, this);
    };
    ElementFinderSync.prototype.assertElementDoesNotExist = function (selector) {
        return selection_1.assertElementDoesNotExist(selector, this);
    };
    ElementFinderSync.prototype.getElementFinder = function () {
        return this.element;
    };
    ElementFinderSync.prototype.getSelectionPath = function () {
        var path = '';
        var args = this.selectionArgs;
        if (args) {
            if (args.rootElement) {
                path += args.rootElement.getSelectionPath() + ' -> ';
            }
            if (args.selector) {
                path += args.selector;
            }
            else if (args.method) {
                path += args.method + '(' + (args.arg || '') + ')';
            }
            if (this.selectionOrdinal > 0 || args.single === false) {
                path += '[' + this.selectionOrdinal + ']';
            }
        }
        return path;
    };
    ElementFinderSync.prototype.reselect = function () {
        var args = this.selectionArgs;
        if (args) {
            var elements = void 0;
            if (args.selector) {
                console.log('(Protractor-sync): Re-selecting stale element: ' + this.getSelectionPath());
                elements = selection_1._getElements(args);
            }
            else if (args.method) {
                console.log('(Protractor-sync): Re-selecting stale element: ' + this.getSelectionPath());
                elements = args.rootElement[args.method](args.arg);
            }
            else {
                console.error('(Protractor-sync): Attempting to re-select stale element, but selection info is incomplete');
            }
            if (Array.isArray(elements)) {
                return elements[this.selectionOrdinal];
            }
            else {
                return elements;
            }
        }
        else {
            console.error('(Protractor-sync): Attempting to re-select stale element, but selection info is missing');
        }
    };
    //Polled waiting
    ElementFinderSync.prototype.waitUntil = function (condition) {
        var _this = this;
        polled_wait_1.polledWait(function () {
            var val = _this.is(condition);
            return { data: null, keepPolling: !val };
        }, function () {
            throw new Error('Timed out(' + config_1.implicitWaitMs + ') waiting for condition: ' + condition);
        });
        return this;
    };
    ElementFinderSync.prototype.waitUntilRemoved = function () {
        var _this = this;
        polled_wait_1.polledWait(function () {
            return { data: null, keepPolling: _this.isPresent() };
        }, function () {
            throw new Error('Timed out(' + config_1.implicitWaitMs + ') waiting for element to be removed');
        });
        return this;
    };
    //ElementFinder methods
    ElementFinderSync.prototype.isPresent = function () {
        return exec_1.exec(this.element.isPresent());
    };
    ElementFinderSync.fromWebElement_ = function (browser, webElem, locator) {
        var _browser = browser instanceof browser_sync_1.BrowserSync ? browser.getBrowser() : browser;
        return new ElementFinderSync(protractor_1.ElementFinder.fromWebElement_(_browser, webElem, locator));
    };
    ElementFinderSync.prototype.evaluate = function (expression) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.evaluate(expression)); });
    };
    ElementFinderSync.prototype.allowAnimations = function (value) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.allowAnimations(value)); });
    };
    ElementFinderSync.prototype.isElementPresent = function (subLocator) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.isElementPresent(subLocator)); });
    };
    ElementFinderSync.prototype.click = function () {
        var _this = this;
        var startTime = new Date().getTime();
        var attempt = function () {
            try {
                exec_1.exec(_this.element.click());
            }
            catch (e) {
                //Unfortunately there's no way to detect this except for checking the string contents.
                var isClickError = /Element is not clickable at point/.test(e.message) || /Other element would receive the click/.test(e.message);
                if (config_1.autoRetryClick && isClickError && new Date().getTime() - startTime < config_1.implicitWaitMs) {
                    console.log('(Protractor-sync): Element (' + _this.getSelectionPath() + ') was covered or unclickable, retrying click.');
                    var flow = ab.getCurrentFlow();
                    flow.sync(setTimeout(flow.add(), config_1.clickRetryIntervalMs)); //We don't need this to retry as quickly
                    return attempt();
                }
                else {
                    throw e;
                }
            }
        };
        this.runWithStaleDetection(function () { return attempt(); });
        return this;
    };
    ElementFinderSync.prototype.sendKeys = function () {
        var _this = this;
        var varArgs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            varArgs[_i] = arguments[_i];
        }
        this.runWithStaleDetection(function () { return exec_1.exec(_this.element.sendKeys.apply(_this.element, varArgs)); });
        return this;
    };
    ElementFinderSync.prototype.getTagName = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getTagName()); });
    };
    ElementFinderSync.prototype.getWebElement = function () {
        return exec_1.exec(this.getElementFinder().getWebElement());
    };
    ElementFinderSync.prototype.getCssValue = function (cssStyleProperty) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getCssValue(cssStyleProperty)); });
    };
    ElementFinderSync.prototype.getAttribute = function (attributeName) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getAttribute(attributeName)); });
    };
    ElementFinderSync.prototype.getText = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getText()); });
    };
    ElementFinderSync.prototype.getSize = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getSize()); });
    };
    ElementFinderSync.prototype.getLocation = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getLocation()); });
    };
    ElementFinderSync.prototype.isEnabled = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.isEnabled()); });
    };
    ElementFinderSync.prototype.isSelected = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.isSelected()); });
    };
    ElementFinderSync.prototype.submit = function () {
        var _this = this;
        this.runWithStaleDetection(function () { return exec_1.exec(_this.element.submit()); });
        return this;
    };
    ElementFinderSync.prototype.clear = function () {
        var _this = this;
        this.runWithStaleDetection(function () { return exec_1.exec(_this.element.clear()); });
        return this;
    };
    ElementFinderSync.prototype.isDisplayed = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.isDisplayed()); });
    };
    ElementFinderSync.prototype.takeScreenshot = function (optScroll) {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.takeScreenshot(optScroll)); });
    };
    //Extras
    ElementFinderSync.prototype.querySelector = function (selector) {
        var _this = this;
        var selectedElement = this.runWithStaleDetection(function () {
            return vars_1.browserSync.executeScript(function (element, _selector) { return element.querySelector(_selector); }, _this, selector);
        }) || undefined;
        if (selectedElement) {
            selectedElement.selectionArgs = {
                rootElement: this,
                method: 'querySelector',
                arg: selector,
                single: true
            };
        }
        return selectedElement;
    };
    ElementFinderSync.prototype.querySelectorAll = function (selector) {
        var _this = this;
        var selectedElements = this.runWithStaleDetection(function () {
            return vars_1.browserSync.executeScript(function (element, _selector) { return element.querySelectorAll(_selector); }, _this, selector);
        });
        selectedElements.forEach(function (element, i) {
            element.selectionArgs = {
                rootElement: _this,
                method: 'querySelectorAll',
                arg: selector
            };
            element.selectionOrdinal = i;
        });
        return selectedElements;
    };
    ElementFinderSync.prototype.getOuterHtml = function () {
        var _this = this;
        return this.runWithStaleDetection(function () {
            return vars_1.browserSync.executeScript(function (element) { return element.outerHTML; }, _this);
        });
    };
    ElementFinderSync.prototype.getInnerHtml = function () {
        var _this = this;
        return this.runWithStaleDetection(function () {
            return vars_1.browserSync.executeScript(function (element) { return element.innerHTML; }, _this);
        });
    };
    ElementFinderSync.prototype.serialize = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.serialize()); });
    };
    ElementFinderSync.prototype.getId = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return exec_1.exec(_this.element.getId()); });
    };
    ElementFinderSync.prototype.scrollIntoView = function () {
        var _this = this;
        this.runWithStaleDetection(function () { return vars_1.browserSync.executeScript(function (element) {
            element.scrollIntoView();
        }, _this); });
        return this;
    };
    ElementFinderSync.prototype.sendEnterKey = function () {
        this.sendKeys(selenium_webdriver_1.Key.ENTER);
        return this;
    };
    ElementFinderSync.prototype.sendTabKey = function () {
        this.sendKeys(selenium_webdriver_1.Key.TAB);
        return this;
    };
    //JQuery methods
    ElementFinderSync.prototype.executeJQueryElementMethod = function (method, arg) {
        var _this = this;
        var attempt = function () {
            _this.injectjQuery(); //Automatically inject jQuery into the page for testing sites that don't use it
            return vars_1.browserSync.executeScript(function (element, _method, _arg) {
                var $ = window.jQuery;
                if (!$) {
                    return '!!jquery not present!!';
                }
                var result = _arg ? $(element)[_method](_arg) : $(element)[_method]();
                if (result instanceof window.jQuery) {
                    return result.toArray();
                }
                else {
                    return result;
                }
            }, _this, method, arg);
        };
        var result = this.runWithStaleDetection(function () { return attempt(); });
        if (result === '!!jquery not present!!') {
            throw Error('jQuery not present, unable to continue');
        }
        if (Array.isArray(result)) {
            return result.map(function (elementFinder, i) {
                //TODO: clean up
                elementFinder.selectionArgs = {
                    rootElement: _this,
                    method: method,
                    arg: arg
                };
                elementFinder.selectionOrdinal = i;
                return elementFinder;
            });
        }
        else {
            return result;
        }
    };
    ElementFinderSync.prototype.injectjQuery = function () {
        var jQuery = vars_1.browserSync.executeScript(function () {
            return !!window.jQuery;
        });
        if (!jQuery) {
            var jquerySource = fs.readFileSync(config_1.jQueryLocation, 'utf8');
            vars_1.browserSync.executeScript(function (_jquerySource) {
                /* tslint:disable-next-line:no-eval */
                eval(_jquerySource);
                window.$.noConflict();
            }, jquerySource);
        }
    };
    ElementFinderSync.prototype.closest = function (selector) {
        return this.executeJQueryElementMethod('closest', selector)[0];
    };
    ElementFinderSync.prototype.hasClass = function (className) {
        var _this = this;
        return this.runWithStaleDetection(function () { return vars_1.browserSync.executeScript(function (element, _className) {
            return element.classList.contains(_className);
        }, _this, className); });
    };
    ElementFinderSync.prototype.isFocused = function () {
        var _this = this;
        return this.runWithStaleDetection(function () { return vars_1.browserSync.executeScript(function (element) {
            return document.activeElement === element;
        }, _this); });
    };
    ElementFinderSync.prototype.innerHeight = function () {
        return this.executeJQueryElementMethod('innerHeight');
    };
    ElementFinderSync.prototype.innerWidth = function () {
        return this.executeJQueryElementMethod('innerWidth');
    };
    ElementFinderSync.prototype.is = function (selector) {
        return this.executeJQueryElementMethod('is', selector);
    };
    ElementFinderSync.prototype.outerHeight = function (includeMargin) {
        return this.executeJQueryElementMethod('outerHeight', includeMargin);
    };
    ElementFinderSync.prototype.outerWidth = function (includeMargin) {
        return this.executeJQueryElementMethod('outerWidth', includeMargin);
    };
    ElementFinderSync.prototype.next = function (selector) {
        return this.executeJQueryElementMethod('next', selector)[0];
    };
    ElementFinderSync.prototype.offset = function () {
        return this.executeJQueryElementMethod('offset');
    };
    ElementFinderSync.prototype.parent = function (selector) {
        return this.executeJQueryElementMethod('parent', selector)[0];
    };
    ElementFinderSync.prototype.parents = function (selector) {
        return this.executeJQueryElementMethod('parents', selector);
    };
    ElementFinderSync.prototype.position = function () {
        return this.executeJQueryElementMethod('position');
    };
    ElementFinderSync.prototype.prev = function (selector) {
        return this.executeJQueryElementMethod('prev', selector)[0];
    };
    ElementFinderSync.prototype.prop = function (name) {
        return this.executeJQueryElementMethod('prop', name);
    };
    ElementFinderSync.prototype.scrollLeft = function () {
        return this.executeJQueryElementMethod('scrollLeft');
    };
    ElementFinderSync.prototype.scrollTop = function () {
        return this.executeJQueryElementMethod('scrollTop');
    };
    ElementFinderSync.prototype.runWithStaleDetection = function (func) {
        var _this = this;
        var attempt = function () {
            try {
                return func();
            }
            catch (e) {
                if (e.name === 'StaleElementReferenceError') {
                    var reselected = _this.reselect();
                    if (reselected) {
                        _this.element = reselected.getElementFinder();
                        return attempt();
                    }
                }
                throw e;
            }
        };
        return attempt();
    };
    return ElementFinderSync;
}());
exports.ElementFinderSync = ElementFinderSync;
//# sourceMappingURL=element-finder-sync.js.map