"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ab = require("asyncblock");
var config_1 = require("./config");
var exec_1 = require("./exec");
function polledExpect(func, waitTimeMS) {
    var timeout = waitTimeMS || config_1.implicitWaitMs;
    var startTime = new Date().getTime();
    var matchers = Object.create(null);
    var flow = ab.getCurrentFlow();
    var originalContext = new Error();
    var isNot = false;
    var matcherCalled = false;
    setTimeout(function () {
        if (!matcherCalled) {
            originalContext.message = 'polledExpect() was called without calling a matcher';
            console.error(originalContext.stack);
            //There's no way to fail the current test because the afterEach has already run by this point
            //Exiting the process is the only way to guarantee a developer will notice the problem
            process.exit(1);
        }
    });
    Object.keys(jasmine.matchers).forEach(function (key) {
        matchers[key] = function (expected) {
            matcherCalled = true;
            var passed = false;
            do {
                var actual = exec_1.exec(func());
                var result = jasmine.matchers[key](jasmine.matchersUtil, null).compare(actual, expected);
                passed = result.pass;
                if (isNot) {
                    passed = !passed;
                }
                if (!passed) {
                    if (new Date().getTime() - startTime <= timeout) {
                        flow.sync(setTimeout(flow.add(), config_1.retryIntervalMs));
                    }
                    else {
                        var message = result.message;
                        if (!message) {
                            message = jasmine.matchersUtil.buildFailureMessage(key, isNot, actual, expected);
                        }
                        throw new Error(message);
                    }
                }
            } while (!passed);
        };
    });
    Object.defineProperty(matchers, 'not', {
        get: function () {
            isNot = true;
            return matchers;
        }
    });
    return matchers;
}
exports.polledExpect = polledExpect;
//Expose global variable so callers can call "polledExpect" similar to just calling "expect"
global.polledExpect = polledExpect;
//# sourceMappingURL=polled-expect.js.map