"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ab = require("asyncblock");
var config_1 = require("./config");
/**
 * Executes a function repeatedly until it returns a value other than undefined. Waits RETRY_INTERVAL ms between function calls.
 *
 * @param fn The function to execute repeatedly
 * @param onTimeout An optional function to call when fn doesn't return a defined value before IMPLICIT_WAIT_MS.
 *                  If this is not specified then a generic exception will be raised.
 * @param waitTimeMs Override the amount of time to wait before timing out
 * @returns {any} The last value the function returned, as long as it did not time out
 */
function polledWait(fn, onTimeout, waitTimeMs) {
    var startTime = new Date();
    var timeout = waitTimeMs != null ? waitTimeMs : config_1.implicitWaitMs;
    var result;
    var flow = ab.getCurrentFlow();
    while (true) {
        if (result == null || new Date().getTime() - startTime.getTime() < timeout) {
            result = fn();
            if (result.keepPolling) {
                flow.sync(setTimeout(flow.add(), config_1.retryIntervalMs)); //Wait a bit before checking again
            }
            else {
                break;
            }
        }
        else {
            if (onTimeout) {
                onTimeout(result.data);
                break;
            }
            else {
                throw new Error('Timed out(' + timeout + ') waiting for function: ' + fn.name);
            }
        }
    }
    return result.data;
}
exports.polledWait = polledWait;
//# sourceMappingURL=polled-wait.js.map