import { ElementFinderSync } from './element-finder-sync';
/**
 * Selects an element or elements. This function is not intended to be called directly (use findVisible, findVisibles, findElement, etc.)
 *
 * @param args.selector A css selector or locator used to select the element(s)
 * @param args.shouldExist True if we expect the element to be found, false otherwise
 * @param args.single True if we're only selecting one element, false otherwise
 * @param args.requireVisible True if the elements must be visible
 * @param args.rootElement Constrain selection to descendants of this element only
 * @returns {protractor.ElementFinder[]}
 * @private
 */
export declare function _getElements(args: {
    selector: any;
    shouldExist: boolean;
    single: boolean;
    requireVisible: boolean;
    rootElement: ElementFinderSync;
    poll: boolean;
}): any;
/**
 * Asserts that an element is NOT present. Polls in order to give the element time to disappear from the DOM.
 * If time expires and the element is still present, an error will be thrown.
 * @param selector A CSS selector or element locator
 * @param rootElement If specified, only search for descendants of this element
 * @returns true if there are no matching elements
 */
export declare function assertElementDoesNotExist(selector: any, rootElement?: ElementFinderSync): boolean;
/**
 * Finds a single visible instance of an element. If more than one visible elements match the locator,
 * and error is thrown. If no visible elements match the locator, an error is thrown. Implicitly waits until there is exactly one
 * visible element.
 *
 * @param selector A CSS selector or element locator
 * @param rootElement If specified, only find descendants of this element
 * @returns {ElementFinder}
 */
export declare function findVisible(selector: any, rootElement?: ElementFinderSync): ElementFinderSync;
/**
 * Finds multiple visible elements. If no visible elements match the locator, an error is thrown.
 * Implicitly waits until at least one visible element is found.
 *
 * @param selector A CSS selector or element locator
 * @param rootElement If specified, only find descendants of this element
 * @returns {ElementFinder[]}
 */
export declare function findVisibles(selector: any, rootElement?: ElementFinderSync): ElementFinderSync[];
/**
 * Finds a single element. If no elements match the locator, an error is thrown.
 * Implicity waits until one element is found.
 *
 * @param selector A CSS selector or element locator
 * @param rootElement If specified, only find descendants of this element
 * @returns {ElementFinder}
 */
export declare function findElement(selector: any, rootElement?: ElementFinderSync): ElementFinderSync;
/**
 * Finds multiple elements. If no elements match the locator, an error is thrown.
 * Implicitly waits until at least one element is found.
 *
 * @param selector A CSS selector or element locator
 * @param rootElement If specified, only find descendants of this element
 * @returns {ElementFinder}
 */
export declare function findElements(selector: any, rootElement?: ElementFinderSync): ElementFinderSync[];
